using System;
using System.Collections.Generic;
using System.Net.Mime;
using System.Security.Cryptography;
using FarseerPhysics.Dynamics.Joints;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using Silk.NET.SDL;
using Color = Jypeli.Color;

namespace KelaCombatTesti;

/// @author Sakari
/// @version 17.02.2025
/// <summary>
/// 
/// </summary>
    public class KelaCombatTesti : PhysicsGame
    {
        PlatformCharacter pelaaja;
        PhysicsObject Pesismaila;
        PlatformCharacter Kelabotti;
        private Image KelabottiKuva = LoadImage("Kelabotti");
        private Image PelaajaKuva = LoadImage("Pelaaja.png");
        
        //IntMeter edustaa muuttujia, jotka voivat kasvaa tai vähentyä tietyllä tavalla 
        IntMeter pisteet;  //pelaajan keräämät pisteet 
        DoubleMeter pelaajanelamaLaskuri; //pelaajan elämät
        IntMeter Kelabotinelamalaskuri; //kelabotin elämät
        
        //Label = tekstielementti, joka näytetään pelissä 
        Label pisteNaytto; //pisteiden näyttäminen ruudulla 

        
        
      
        /// <summary>
        /// 
        /// </summary>
        public override void Begin()
        {
            LuoKentta();
            LuoKelabotti();
            LuoElamalaskuri();
            
            
            LuoPistelaskuri(); // Alustetaan pistelaskuri 
            
            
            //Ajastin, joka lisää uuden vihollisen joka 5. sekunti 
            Timer.CreateAndStart(5.0, LisaaVihollinen); 
            
          
            
            //kelabotin elämien väheneminen VahingoitaKelabottia aliohjelman mukaisesti
            //AddCollisionHandler(Kelabotti, "Pesismaila", VahingoitaKelabottia);

        
            Camera.Follow(pelaaja);
            
            Keyboard.Listen(Key.A, ButtonState.Down, LiikuVasemmalle, null, new Vector());
            Keyboard.Listen(Key.D, ButtonState.Down, LiikuOikealle, null, new Vector());
            Keyboard.Listen(Key.W, ButtonState.Down, Hyppaa, null, new Vector());
            //Keyboard.Listen(Key.S, ButtonState.Down, LiikutaPelaajaa, null, new Vector(0, -1000));
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
            
            Keyboard.Listen(Key.Space, ButtonState.Pressed, LuoPesismaila, "", new Vector(90, 10), pelaaja);
           

            
        
            void LiikuVasemmalle(Vector vektori)
            {
                pelaaja.Walk(-100);
            }

            void LiikuOikealle(Vector vektori)
            {
                pelaaja.Walk(100);
            }
            
            void Hyppaa(Vector vektori)
            {
                pelaaja.Jump(300.0);
            }
        }
        
        
        /// <summary>
        /// Tässä luodaan pelikenttä
        /// </summary>
        void LuoKentta()
        {
         
            Level.BackgroundColor = Color.Pink;
            Gravity = new Vector(0, -981.0);
            
            PhysicsObject vasenReuna = Level.CreateLeftBorder();
            vasenReuna.Restitution = 0.0;
            vasenReuna.KineticFriction = 0.0;
            vasenReuna.IsVisible = false;

            PhysicsObject oikeaReuna = Level.CreateRightBorder();
            oikeaReuna.Restitution = 0.0;
            oikeaReuna.KineticFriction = 0.0;
            oikeaReuna.IsVisible = false;

            PhysicsObject ylaReuna = Level.CreateTopBorder();
            ylaReuna.Restitution = 0.0;
            ylaReuna.KineticFriction = 0.0;
            ylaReuna.IsVisible = false;

            PhysicsObject alaReuna = Level.CreateBottomBorder();
            alaReuna.Restitution = 1.0;
            alaReuna.IsVisible = true;
            alaReuna.KineticFriction = 0.0;
            alaReuna.Color = Color.Gray;


            pelaaja = LuoPelaaja(500.0, 0.0);

        }
        
        /// <summary>
        /// Aliohjelma, jossa luodaan pelaaja
        /// </summary>
        /// <param name="x">pelaajan x-koordinaatti</param>
        /// <param name="y">pelaajan y-koordinaatti</param>
        /// <returns>pelaaja</returns>
        PlatformCharacter LuoPelaaja(double x, double y)
        {
            PlatformCharacter pelaaja = new PlatformCharacter(50.0, 100.0);
            pelaaja.Image = PelaajaKuva;
            pelaaja.Mass = 1.0;
            pelaaja.Shape = Shape.Rectangle;
            pelaaja.Color = Color.Black;
            pelaaja.X = x;
            pelaaja.Y = y;
            // Törmäyksen käsittelijä, mitä tapahtuu, kun pelaaja osuu kelabotiin 
            //Siis pelaajan elämien väheneminen VahnigoitaPelaajaa-aliohjelman mukaisesti 
            AddCollisionHandler(pelaaja, "Kelabotti", VahingoitaPelaajaa);
            
            Add(pelaaja);
            
            return pelaaja;
        }
        
        /// <summary>
        /// Aliohjelma, joka luo pelaajalle elämälaskurin ja sen nimen: "Mielenterveys"
        /// </summary>
        void LuoElamalaskuri()
        {
            pelaajanelamaLaskuri = new DoubleMeter(100.0);
            pelaajanelamaLaskuri.MaxValue = 100;
            pelaajanelamaLaskuri.LowerLimit += ElamaLoppui;

            ProgressBar elamapalkki = new ProgressBar(150, 18);
            elamapalkki.X = Screen.Left + 110;
            elamapalkki.Y = Screen.Bottom + 50;
            elamapalkki.BindTo(pelaajanelamaLaskuri);
            elamapalkki.Color = Color.Red;
            elamapalkki.BarColor = Color.Green;
            elamapalkki.BorderColor = Color.Black;
            Label tekstikentta = new Label(200, 10, "Mielenterveys");
            tekstikentta.X = Screen.Left + 100;
            tekstikentta.Y = Screen.Bottom + 70;
            tekstikentta.Font = new Font(20);
            Add(tekstikentta);
            Add (elamapalkki);

        }

       
        /// <summary>
        /// Aliohjelma, joka käsittelee pelaajan kuoleman ja ilmoittaa pelin loppumisen
        /// </summary>
        void ElamaLoppui()
        {
            Label Kuolema = new Label(800, 60, "Toimeentulohakemus hylätty.");
            Kuolema.Y = 140;
            Kuolema.Font = new Font(60);
            Kuolema.TextColor = Color.Red;
            //Kuolema.Color = Color.Black;
            Add(Kuolema);
        }
        

        /// <summary>
        /// Tässä luodaan uusi fysiikkaolio, Pesismaila
        //Tämä toimii pelaajan lyöntinä (samalla määritellään ase.
        /// </summary>
        /// <param name="koko">Pesismailan koko</param>
        /// <param name="pelaaja">kohde, johon Pesismaila liitetään, eli pelaaja</param>
        void LuoPesismaila(Vector koko, PlatformCharacter pelaaja)
        {
            PhysicsObject Pesismaila = new PhysicsObject(koko.X, koko.Y);
            Pesismaila.Position = pelaaja.Position + new Vector(80*pelaaja.FacingDirection.GetVector().X, 10);
            Pesismaila.CollisionIgnoreGroup = 1;
            Pesismaila.Mass = 1;
            Pesismaila.Tag = "Pesismaila";
            Add(Pesismaila);
            Pesismaila.LifetimeLeft = TimeSpan.FromSeconds(0.075);
            Pesismaila.IgnoresGravity = true;
            
            //PISTEET ei elämät
            AddCollisionHandler(Pesismaila, "Kelabotti", KunVihollinenTuhoutuu);
            
        }

        
        /// <summary>
        /// Tämä funktio luo Kelabotti-vihun ja sille annetaan pelaajan seuraamis aivot
        /// </summary>
        /// <returns>Kelabotti</returns>
        PlatformCharacter LuoKelabotti()
        {
            PlatformCharacter Kelabotti = new PlatformCharacter(80.0, 80.0);
            Kelabotti.Color = Color.Orange;
            Kelabotti.Image = KelabottiKuva;
            Kelabotti.MirrorImage();
            Kelabotinelamalaskuri = new IntMeter(5); // Alussa on 5 elämää 
            Kelabotti.Tag = "Kelabotti";
            Add(Kelabotti);
            
            
            FollowerBrain seuraajanAivot = new FollowerBrain(pelaaja);
            seuraajanAivot.Speed = 100;

            Kelabotti.Brain = seuraajanAivot;
            return Kelabotti;
            
        }
        /// <summary>
        /// Lisää Kelabotteja
        /// </summary>
        void LisaaVihollinen() 
        { 
            Random rand = new Random(); 
            double x = rand.Next(-400, 400); // Satunnainen X-koordinaatti 
            double y = rand.Next(-200, 200); // Satunnainen Y-koordinaatti 
            LuoKelabotti(); // Luodaan uusi vihollinen satunnaiseen paikkaan
        } 
        /// <summary>
        /// Tämä aliohjelma käsittlee sen, kun pelaaja osuu Kelabottiin
        /// </summary>
        /// <param name="pelaaja">pelaajahahmo</param>
        /// <param name="Kelabotti">pelin vihu; Kelabotti</param>
        void VahingoitaPelaajaa(PhysicsObject pelaaja, PhysicsObject Kelabotti)
        {
            int luku = RandomGen.NextInt(19, 34);
            pelaajanelamaLaskuri.Value = pelaajanelamaLaskuri - luku; // Vähentää pelaajan elämää satunnaisen määrän
            //MessageDisplay.Add("Osuma! Elämät jäljellä: " + pelaajanelamaLaskuri.Value); 
            
            if (pelaajanelamaLaskuri.Value <= 0) //jos laskuri menee nollaan... 
            { 
                pelaaja.Destroy(); //...ja pelaaja tuhotaan
                //Pesismaila.Destroy();//...myös pesismaila
            } 
            
        } 
        
        //tää ei toimi vielä
        void VahingoitaKelabottia(PhysicsObject Kelabotti, PhysicsObject Pesismaila) 
        { 
            Kelabotinelamalaskuri.Value--; // Vähentää kelabotin elämää yhdellä 
            if (Kelabotinelamalaskuri.Value <= 0) //jos laskuri menee nollaan... 
            { 
                Kelabotti.Destroy(); //...ja kelabotti tuhotaan
            } 
        } 
        /// <summary>
        /// Luo pistelaskurin pisteille, joita saa kun hakkaa kelabotteja
        /// </summary>
        void LuoPistelaskuri() 
        { 
            
            Label opintoNopat = new Label(150, 50, "Opintonopat:");
            opintoNopat.X = Screen.Left + 100;
            opintoNopat.Y = Screen.Bottom + 93;
            opintoNopat.Font = new Font(20);
            opintoNopat.TextColor = Color.Black;
            Add(opintoNopat);
            
            pisteet = new IntMeter(0); // Aloitetaan nollasta 
            pisteNaytto = new Label(); 
            pisteNaytto.X = Screen.Left + 170;  
            pisteNaytto.Y = Screen.Bottom + 95;  
            pisteNaytto.TextColor = Color.OrangeRed; 
            pisteNaytto.Font = new Font(20); 
            pisteNaytto.BindTo(pisteet); // Päivitetään teksti automaattisesti 
            Add(pisteNaytto);
            
            
        } 
        
        /// <summary>
        /// Käsittelee kelabotin tuhoamisen ja antaa siitö pisteitä
        /// </summary>
        /// <param name="pelaaja">pelaajahahmo</param>
        /// <param name="kelabotti">pelin vihu; Kelabotti</param>
        void KunVihollinenTuhoutuu(PhysicsObject pelaaja, PhysicsObject kelabotti) 
        { 
            pisteet.Value += 100; // Lisää esim. 100 pistettä, kun botti tuhotaan
            kelabotti.Destroy(); // Poistaa botin näytöltä
        } 
        
    }

        
    


